use std::collections::HashMap;
use std::process::{Command, Stdio};
use std::io::Write;

pub mod passwords;
pub mod totp;
pub mod hotp;

pub mod libnitrokey;
use libnitrokey::*;

pub enum TokenType {
    TOTP,
    HOTP,
    PASSWORD
}

pub fn get_tokens(token_type: TokenType) -> HashMap<String, u8> {
    let _token_size = match token_type {
        TokenType::TOTP => 15,
        TokenType::PASSWORD => 16,
        TokenType::HOTP => 3
    };
    match token_type {
        TokenType::TOTP => {
            totp::get_tokens()
        },
        TokenType::HOTP => {
            hotp::get_tokens()
        },
        TokenType::PASSWORD => {
            passwords::get_tokens()
        }
    }
}

pub fn print_tokens(token_type: TokenType) {
    let tokens = get_tokens(token_type);

    for (slot_name, slot_number) in &tokens {
        println!("{} - {}", slot_number, slot_name);
    }
}

pub fn print_token(token_type: TokenType, slot_name: &String) {
   let token: String = match token_type {
       TokenType::TOTP => {
           totp::get_token(slot_name)
       },
       TokenType::PASSWORD => {
           passwords::get_token(slot_name)
       },
       TokenType::HOTP => {
           hotp::get_token(slot_name)
       }
    };
    println!("{}", token);
    save_to_clipboard(token);
}


pub fn init_libnitrokey() {
    unsafe {
        NK_login_auto();
    }
}

pub fn close_libnitrokey() {
    unsafe {
        NK_lock_device();
        NK_logout()
    };
}

pub fn save_to_clipboard(slot_code: String) {
    let mut child = Command::new("xclip")
        .arg("-selection")
        .arg("clipboard")
        .stdin(Stdio::piped())
        .stdout(Stdio::null())
        .spawn()
        .expect("Failed to spawn xclip child process");

    let xclip_stdin = child.stdin.as_mut().expect("Failed to open stdin");
    xclip_stdin.write_all(slot_code.as_bytes()).expect("Failed to write to stdin");

}
